
import unittest
import yaml
import os.path
from jacobsjsonschema.draft7 import Validator


example_stinger_path = os.path.normpath(os.path.join(os.path.dirname(os.path.abspath(__file__)), "../example_interfaces"))
schema_path = os.path.normpath(os.path.join(os.path.dirname(os.path.abspath(__file__)), "../schemas/schema.yaml"))


class SchemaValidation:

    def test_validates(self):
        with open(schema_path) as fp:
            validator = Validator(yaml.load(fp, Loader=yaml.CSafeLoader), lazy_error_reporting=True)
        
        param_only_path = os.path.join(example_stinger_path, self.directory, self.stinger_name)
        with open(param_only_path) as fp:
            interface = yaml.load(fp, Loader=yaml.CSafeLoader)
        is_valid = validator.validate(interface)
        if not is_valid:
            for err in validator.get_errors():
                print(err)
            self.assertEqual(len(validator.get_errors()), 0)
        self.assertTrue(is_valid)


class TestParamOnlyValidatesAgainstSchema(unittest.TestCase, SchemaValidation):

    def setUp(self):
        self.directory = "signal_only"
        self.stinger_name = "signal_only.stingeripc"


class TestEnumOnlyValidatesAgainstSchema(unittest.TestCase, SchemaValidation):

    def setUp(self):
        self.directory = "enum_only"
        self.stinger_name = "enum_only.stingeripc"




from jacobsjinjatoo import templator as jj2
from jacobsjinjatoo import stringmanip
import os.path
import typer
from typing_extensions import Annotated
from stingeripc import StingerInterface
from rich import print
from pathlib import Path

def main(inname: Annotated[Path, typer.Argument(exists=True, file_okay=True, dir_okay=False, readable=True)], outdir: Annotated[Path, typer.Argument(file_okay=False, dir_okay=True, writable=True, readable=True)]):

    print(f"[bold]Reading:[/bold] {inname}")
    with open(inname, "r") as f:
        stinger = StingerInterface.from_yaml(f)
    params = {
        "stinger": stinger,
    }
    output_dir = Path(outdir).resolve()
    print(f"[bold]Output directory:[/bold] {output_dir}")
    t = jj2.CodeTemplator(output_dir=output_dir)
    t.add_template_dir(
        os.path.join(os.path.dirname(__file__), "../templates", "python")
    )
    for output_file in [
        "pyproject.toml",
    ]:
        print(f"[bold green]Generating:[/bold green] {output_file}")
        t.render_template(f"{output_file}.jinja2", output_file, **params)

    package_name = f"{stringmanip.lower_camel_case(stinger.name).lower()}ipc"
    generated_pkg_dir = output_dir / "src" / package_name
    print(f"[bold]Making Directory:[/bold] {generated_pkg_dir}")
    os.makedirs(generated_pkg_dir, exist_ok=True)
    for output_file in [
        "server.py",
        "client.py",
        "connection.py",
        "__init__.py",
        "method_codes.py",
    ]:
        of = generated_pkg_dir / output_file
        print(f"[bold green]Generating:[/bold green] {of}")
        output = t.render_template(f"{output_file}.jinja2", of, **params)

    t.render_template("interface_types.py.jinja2", f"{generated_pkg_dir}/{stinger.get_enum_module_name()}.py", **params)

    os.makedirs(output_dir / "examples", exist_ok=True)
    for output_file in [
        "client_demo.py",
        "server_demo.py",
    ]:
        of = output_dir / "examples" / output_file
        print(f"[bold green]Generating:[/bold green] {of}")
        output = t.render_template(f"{output_file}.jinja2", of, **params)

def run():
    typer.run(main)

if __name__ == "__main__":
    run()



import os
import sys
from wsgiref.validate import validator
from rich import print
from pathlib import Path
import typer
from typing_extensions import Annotated
from jacobsjinjatoo import templator as jj2
from jacobsjsonschema.draft7 import Validator
import yaml
import yamlloader
from stingeripc.interface import StingerInterface

from . import markdown_generator
from . import python_generator
from . import rust_generator
from . import cpp_generator

app = typer.Typer(help="stinger-ipc generator CLI")


@app.command()
def generate(
    language: Annotated[str, typer.Argument(...)],
    input_file: Annotated[Path, typer.Argument(..., exists=True, file_okay=True, dir_okay=False, readable=True)],
    output_dir: Annotated[Path, typer.Argument(..., file_okay=False, dir_okay=True, writable=True, readable=True)],
):
    """Generate code for a Stinger interface.

    LANGUAGE must be one of: rust, python, markdown, cpp, web, protobuf
    INPUT_FILE is the .stinger.yaml file
    OUTPUT_DIR is the directory that will receive generated files
    """
    lang = language.lower()
    if lang not in ("rust", "python", "markdown", "cpp", "web", "protobuf"):
        raise typer.BadParameter("language must be one of: rust, python, markdown, cpp, web, protobuf")

    if lang == "python":
        # python_generator.main expects Path arguments via typer
        python_generator.main(input_file, output_dir)
    elif lang == "markdown":
        # markdown_generator.main expects Path arguments via typer
        markdown_generator.main(input_file, output_dir)
    elif lang == "web":
        wt = jj2.WebTemplator(output_dir=output_dir)
        ct = jj2.CodeTemplator(output_dir=output_dir)
        wt.add_template_dir(
            os.path.join(os.path.dirname(__file__), "../templates", "html")
        )
        ct.add_template_dir(
            os.path.join(os.path.dirname(__file__), "../templates", "html")
        )
        with open(input_file, "r") as f:
            stinger = StingerInterface.from_yaml(f)
        for output_file in [
            "app.js",
            "styles.css",
        ]:
            ct.render_template(f"{output_file}.jinja2", output_file, stinger=stinger)
        wt.render_template("index.html.jinja2", "index.html", stinger=stinger)
    elif lang == "cpp":
        cpp_generator.main(input_file, output_dir)
    elif lang == "rust":
        rust_generator.main(input_file, output_dir)
    elif lang == "protobuf":
        ct = jj2.CodeTemplator(output_dir=output_dir)
        ct.add_template_dir(
            os.path.join(os.path.dirname(__file__), "../templates", "protobuf")
        )
        with open(input_file, "r") as f:
            stinger = StingerInterface.from_yaml(f)
        ct.render_template("proto.jinja2", f"{stinger.name}.proto", stinger=stinger)
    else:
        raise RuntimeError("Unreachable code reached")

    print(f"Generation for '{lang}' completed.")

@app.command()
def validate(input_file: Annotated[Path, typer.Argument(..., exists=True, file_okay=True, dir_okay=False, readable=True)]):
    """Validate a Stinger interface YAML file.

    INPUT_FILE is the .stinger.yaml file
    """
    schema_file = Path(__file__).parent.parent / "schema" / "schema.yaml"
    schema_obj = yaml.load(schema_file.open("r"), Loader=yamlloader.ordereddict.Loader)
    validator = Validator(schema_obj, lazy_error_reporting=False)

    input_obj = yaml.load(input_file.open("r"), Loader=yamlloader.ordereddict.Loader)
    if result := validator.validate(input_obj):
        print("Validated: ", result)
    else:
        for error in validator.get_errors():
            print(error)
        sys.exit(1)
    sys.exit(0)

@app.command()
def hello():
    print("Hello world")

def run():
    app()

if __name__ == "__main__":
    run()

